	include "p16f877.inc"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;							      ;;
;; - Foobarra - 					      ;;
;; 							      ;;
;; controllo digitale di frequenza di [u]Pulsar tramite Stube ;;
;;							      ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

UNIT		EQU	0x20		; unit	
DEC		EQU	0x21		; decine
CENT		EQU	0x22		; centinaia
VAL		EQU	0x23		; valore di controllo per Stube
TEMP		EQU	0x24	
TEMP2		EQU	0x25
TEMP3		EQU	0x26
MAX		EQU	0X27		; byte di controllo [over|under]flow
					; bit0 -> Unit Overflow
					; bit1 -> Dec Overflow
					; bit2 -> Unit Underflow
					; bit3 -> Dec Underflow

		ORG 	0x0000
		BCF 	PCLATH,3
		BCF 	PCLATH,4
		GOTO 	INIT
	
		ORG 	0x0004
		RETFIE


;; Inizializza LCD e porte

INIT:		BSF	STATUS,RP0
		CLRF	TRISB		; LCD data
		CLRF	TRISC		; Oscillator
		MOVLW	0xF0		
		MOVWF	TRISD		; low-nibble->LCD ctrl, high-nibble->DATA input
		BCF	STATUS,RP0
		BCF 	PORTD,3		; E
		BCF 	PORTD,1		; RS
		BCF 	PORTD,2		; R/W
		CLRF 	PORTB		; DATA	
		MOVLW	0x30		; 8 bit dati, 1 linea, 5*7 pixel	
		;MOVLW 	0x38		; 8 bit dati, 2 linee, 5*7 pixel
		CALL 	L_CMD		
		MOVLW	0x0E		; cursore visibile (0x0C non visibile)
		CALL 	L_CMD
		MOVLW 	0x01		; clear display
		CALL 	L_CMD
		MOVLW 	'F'		; carica la prima riga: "Frequency:"
		CALL 	L_CHAR
		MOVLW 	'r'
		CALL 	L_CHAR
		MOVLW 	'e'
		CALL 	L_CHAR
		MOVLW 	'q'
		CALL 	L_CHAR
		MOVLW 	'u'
		CALL 	L_CHAR
		MOVLW 	'e'
		CALL 	L_CHAR
		MOVLW 	'n'
		CALL 	L_CHAR
		MOVLW 	'c'
		CALL 	L_CHAR
		MOVLW 	'y'
		CALL 	L_CHAR
		MOVLW 	':'
		CALL 	L_CHAR
		MOVLW	0x14		; salta 3 caratteri e scrivi "KHz"
		CALL	L_CMD		
		MOVLW	0x14
		CALL	L_CMD		
		MOVLW	0x14
		CALL	L_CMD		
		MOVLW	'K'
		CALL	L_CHAR
		MOVLW	'H'
		CALL	L_CHAR
		MOVLW	'z'
		CALL	L_CHAR
		MOVLW	0x10
		CALL	L_CMD
		MOVLW	0x10
		CALL	L_CMD
		MOVLW	0x10
		CALL	L_CMD
		

;; Loop principale di acquisizione input

START:		MOVFW	VAL		
		MOVWF	PORTC		; ogni loop carica il valore di F su Stube
		BTFSC	MAX,2		; -1 (salta se underflow)		
		GOTO	INC_ONLY		
		BTFSC	PORTD,4
		GOTO	DEC_U
		BTFSC	MAX,3		; -10 (salta se underflow)
		GOTO	INC_ONLY
		BTFSC	PORTD,5
		GOTO	DEC_D
INC_ONLY:	BTFSC	MAX,0		; +1 (salta se overflow)
		GOTO	START
		BTFSC	PORTD,6
		GOTO	INC_U
		BTFSC	MAX,1		; +10 (salta se overflow)
		GOTO	START		
		BTFSC	PORTD,7		
		GOTO	INC_D
		GOTO 	START


;; Incremento di unit
	
INC_U:		BTFSC	MAX,2		
		CALL	UBLK_UU		; se abilitato disabilita Unit-Underflow
		MOVFW	UNIT
		XORLW	0x09
		ADDLW	0x01
		MOVWF	TEMP
		DECFSZ	TEMP		; se UNIT  a 9 incrementa le decine e si azzera
		GOTO 	ONE_U		; se decine o centinaia sono al massimo valore, entra in overflow
		MOVFW	DEC
		XORLW	0x04
		MOVWF	TEMP
		INCF	TEMP
		DECFSZ	TEMP
		GOTO	CENT_CRTL
		MOVFW	DEC
		XORLW	0x09
		MOVFW	TEMP
		INCF	TEMP
		DECFSZ	TEMP
		GOTO 	NRML
CENT_CRTL:	MOVFW	CENT
		XORLW	0x02
		MOVWF	TEMP
		INCF	TEMP
		DECFSZ	TEMP
		GOTO	NRML
		GOTO	SET_OF
NRML:		CLRF	UNIT
		INCF	VAL
		GOTO	ONE_D
ONE_U:		INCF	VAL		; incrementa le unit di un valore 1 (1 KHz alla volta)
		INCF	UNIT
		CALL	PRINT
		GOTO	START
SET_OF:		BSF	MAX,0
		BSF	MAX,1
		GOTO 	START


;; Incremento di decine 
	
INC_D:		MOVFW	VAL		; incrementa le decine tenendo conto di overflow
		ADDLW	0x0A		; ed eventuali underflow settati da sbloccare
		MOVWF	VAL
INC_D2:		BTFSC	MAX,3
		CALL	UBLK_DU
		MOVFW	DEC
		XORLW	0x04
		ADDLW	0x01
		MOVWF	TEMP
		DECFSZ	TEMP
		GOTO	FORWARD
		GOTO 	TEST_CENT
FORWARD:	MOVFW	DEC
		XORLW	0x09
		ADDLW	0x01
		MOVWF	TEMP
		DECFSZ	TEMP
		GOTO	ONE_D	
		GOTO	ONE_C
ONE_D:		INCF	DEC
		CALL	PRINT
		GOTO 	START
TEST_CENT:	MOVFW	CENT	
		XORLW	0x02
		ADDLW	0x01
		MOVWF	TEMP
		DECFSZ	TEMP
		GOTO 	FORWARD
		GOTO 	OVERFLOW

;; Incremento di centinaia

INC_C:		MOVFW	CENT		
		XORLW	0x02
		ADDLW	0x01
		MOVWF	TEMP
		DECFSZ	TEMP
		GOTO	ONE_C
OVERFLOW:	BSF	MAX,1
		MOVLW	0x0A	
		SUBWF	VAL
		GOTO 	START
ONE_C:		CLRF	DEC
		INCF	CENT
		CALL	PRINT
		GOTO 	START


;; Decremento di unit
	
DEC_U:		BTFSC	MAX,0		
		CALL	UBLK_UO
		MOVFW	UNIT
		ADDLW	0x01
		MOVWF	TEMP
		DECFSZ	TEMP
		GOTO 	ONE_DU
		INCF	DEC
		DECFSZ	DEC
		GOTO	NORMAL
		INCF	CENT	
		DECFSZ	CENT
		GOTO	NORMAL	
		GOTO	SET_UF
NORMAL:		MOVLW	0x09		
		MOVWF	UNIT
		DECF	VAL
		GOTO	DEC_D2
ONE_DU:		DECF	UNIT
		DECF	VAL
		CALL	PRINT
		GOTO	START
SET_UF:		BSF	MAX,2
		BSF	MAX,3
		GOTO 	START
	

;; Decremento di decine

DEC_D:		MOVLW	0x0A		
		SUBWF	VAL
DEC_D2:		BTFSC	MAX,1
		CALL	UBLK_DO
		MOVFW	DEC
		ADDLW	0x01
		MOVWF	TEMP
		DECFSZ	TEMP
		GOTO	ONE_DD
		GOTO	DEC_C
ONE_DD:		DECF	DEC
		CALL	PRINT
		GOTO 	START
			

;; Decremento di centiaia

DEC_C:		MOVFW	CENT		
		ADDLW	0x01
		MOVWF	TEMP
		DECFSZ	TEMP
		GOTO	ONE_CD
		MOVFW	VAL	
		ADDLW	0x0A
		MOVWF	VAL
		BSF	MAX,3
		GOTO 	START
ONE_CD:		MOVLW	0x09
		MOVWF	DEC	
		DECF	CENT
		CALL	PRINT
		GOTO 	START
	

;; Routine di sbloccaggio di [over/under]flow

UBLK_UO:	BCF	MAX,0		; unit overflow
		RETURN

UBLK_DO:	BCF	MAX,1		; unit+dec overflow
		CALL	UBLK_UO
		RETURN

UBLK_UU:	BCF	MAX,2		; unit underflow
		RETURN

UBLK_DU:	BCF	MAX,3		; unit+dec underflow
		CALL	UBLK_UU
		RETURN


;; Stampa il valore di frequenza attuale

PRINT:		MOVLW	0x10		; ritorna di 3 caratteri
		CALL	L_CMD
		MOVLW	0x10
		CALL	L_CMD
		MOVLW	0x10
		CALL	L_CMD
P_CENT:		MOVFW	CENT
		MOVWF	TEMP
		INCF	TEMP
		MOVFW	CENT	
		DECFSZ	TEMP	
		GOTO	P_DEC
		CALL	EMPTY
		GOTO	S_DEC
P_DEC:		ADDLW	0x30
		CALL	L_CHAR
S_DEC:		MOVFW	DEC
		MOVWF	TEMP
		MOVFW	CENT
		ADDWF	TEMP
		INCF	TEMP
		MOVFW	DEC	
		DECFSZ	TEMP	
		GOTO	P_UNIT
		CALL	EMPTY
		GOTO	S_UNIT
P_UNIT:		ADDLW	0x30
		CALL	L_CHAR
S_UNIT:		MOVFW	UNIT
		ADDLW	0x30
		CALL	L_CHAR	
		;CALL	DELAY4
		RETURN
	
EMPTY:		MOVLW	0x00		; stampa un carattere nullo
		CALL	L_CHAR
		RETURN
	

;; Routine di stampa caratteri sul display

L_CHAR:		BSF 	PORTD,1		; setta DATAOUT
		BCF 	PORTD,2		; setta WRITE
		MOVWF 	PORTB		; dato
		BSF 	PORTD,3		; abilita l'LCD ad acquisire
		CALL	DELAY
		BCF 	PORTD,3		; disabilita
		RETURN
	

;; Routine di lancio comandi al display

L_CMD:		BCF 	PORTD,1		; setta CMDOUT
		BCF 	PORTD,2		; setta WRITE
		MOVWF 	PORTB
		BSF 	PORTD,3		; grubba
		CALL	DELAY
		BCF 	PORTD,3		; ok
		RETURN
	

;; Routing di controllo dello stato di BUSY del display (per ora non utilizzata)
	
W_BUSY:		BSF	STATUS,RP0	; sequenza test-busy
		MOVLW	0xFF
		MOVWF	TRISB
		BCF	STATUS,RP0
B_TEST:		BSF	PORTD,2		
		BCF	PORTD,1
		BSF	PORTD,3
		NOP	
		NOP
		BTFSS	PORTB,7
		GOTO	W_ESC
		BCF	PORTD,3
		CALL	DELAY
		GOTO	W_BUSY
W_ESC:		BCF	PORTD,3
		BSF	STATUS,RP0
		CLRF	TRISB
		BCF	STATUS,RP0
		RETURN
	

;; Delay di comandi e dati per l'LCD: circa 24 us

DELAY:		MOVLW	0x30
		MOVWF	TEMP
LOOP:		DECFSZ	TEMP
		GOTO 	LOOP
		RETURN	
	

;; Delay per l'input da pulsante: circa 0.25 s

DELAY4:		MOVLW	0x04	
		MOVWF	TEMP
LOOP4:		DECFSZ	TEMP
		GOTO	DELAY3
		RETURN
	
DELAY3:		MOVLW	0xFF
		MOVWF	TEMP3
LOOP3:		DECFSZ	TEMP3
		GOTO	DELAY2
		GOTO	LOOP4
	
DELAY2:		MOVLW	0xFF
		MOVWF	TEMP2
LOOP2		DECFSZ	TEMP2
		GOTO 	LOOP2
		GOTO	LOOP3
	

		END
